Library Barcode;

{$mode objfpc}{$H+}

// Barcode als Grafik:
// https://forum.lazarus.freepascal.org/index.php?topic=31978.0
// DLL mit Grafik:
// https://forum.lazarus.freepascal.org/index.php?topic=30463.0

Uses
  Classes, Interfaces, Graphics, SysUtils, lbc_datamatrix, lbc_render, zint;

Var FQR: PointerTo_zint_symbol;
    FText: UTF8String;
    FLastErrorString: UTF8String;
    FBackgroundColor: TColor;
    FForegroundColor: TColor;
    FStrictSize: Boolean;

Const sVersion: String = 'V21507';

procedure ZBarcode_Delete(symbol: PointerTo_zint_symbol);
var
  l: PointerTo_zint_render_line;
  line: PointerTo_zint_render_line;
  s: PointerTo_zint_render_string;
  TheString: PointerTo_zint_render_string;
begin
  if Boolean(symbol^.bitmap <> nil) then
  begin
    FreeMem (symbol^.bitmap);
  end;
  if Boolean(symbol^.rendered <> nil) then
  begin
    line := symbol^.rendered^.lines;
    while Assigned(line) do
    begin
      l := line;
      line := line^.next;
      FreeMem (l);
    end;
    TheString := symbol^.rendered^.strings;
    while Assigned(TheString) do
    begin
      s := TheString;
      TheString := TheString^.next;
      FreeMem (s^.text);
      FreeMem (s);
    end;
    FreeMem (symbol^.rendered);
  end;
  FreeMem (symbol);
end;

procedure strcpy(const aText: pchar; const aText2: pchar);
begin
  move(aText2^,aText^,sysutils.strlen(aText2)+1);
end;

procedure strcpy(var aText: array of char; const aText2: pchar);
begin
  move(aText2^,aText[0],sysutils.strlen(aText2)+1);
end;

function ZBarcode_Create(): PointerTo_zint_symbol;
var
  symbol: PointerTo_zint_symbol;
  i: Integer;
begin
  symbol := GetMem (SizeOf (zint_symbol));
  if not Assigned(symbol) then
  begin
    exit (nil);
  end;
  FillByte(symbol^, SizeOf (zint_symbol),0);
  symbol^.symbology := BARCODE_DATAMATRIX;
  symbol^.height := 0;
  symbol^.whitespace_width := 0;
  symbol^.border_width := 0;
  symbol^.output_options := 0;
  symbol^.rows := 0;
  symbol^.width := 0;
  strcpy (symbol^.fgcolour, '000000');
  strcpy (symbol^.bgcolour, 'ffffff');
  strcpy (symbol^.outfile, '');
  symbol^.scale := 1.0;
  symbol^.option_1 := -1;
  symbol^.option_2 := 0;
  symbol^.option_3 := DM_SQUARE;//928;
  symbol^.show_hrt := 1;
  symbol^.input_mode := DATA_MODE;
  strcpy (symbol^.primary, '');
  FillByte (symbol^.encoded_data[0][0], SizeOf (symbol^.encoded_data),0);
  i := 0;
  while i < 178 do
  begin
    symbol^.row_height[i] := 0;
    Inc (i);
  end;
  symbol^.bitmap := nil;
  symbol^.bitmap_width := 0;
  symbol^.bitmap_height := 0;
  exit (symbol);
end;

procedure Generate; // TBarcodeDataMatrix
var
  ErrorCode: integer;
begin
  if Assigned(FQR) then begin
    ZBarcode_Delete(FQR);
    FQR:=nil;
  end;
  if Length(FText)>0 then begin
    FQR:=ZBarcode_Create();
    with FQR^ do begin
      border_width:=1;
    end;
    ErrorCode:=dmatrix(FQR,@FText[1],Length(FText));
    if ErrorCode<>0 then begin
      FLastErrorString:=FQR^.errtxt;
      exit;
    end;
  end;
end;

///////////////////////////////////////

procedure intfPaintOnCanvas(const aTargetCanvas: TCanvas; const aRect: TRect);
var
  ErrorCode: integer;
  Line: PointerTo_zint_render_line;
  BaseX,BaseY: integer;
  X,Y: integer;
  procedure ClearBackground;
  begin
    aTargetCanvas.Brush.Color:=FBackgroundColor;
    aTargetCanvas.FillRect(aRect);
  end;
begin
  if not aTargetCanvas.HandleAllocated then exit;
  if not Assigned(FQR) then begin
    ClearBackground;
    exit;
  end;
  if not Assigned(FQR^.rendered) then begin
    X:=aRect.Right-aRect.Left+1;
    Y:=aRect.Bottom-aRect.Top+1;
    if not FStrictSize then begin
      ErrorCode:=render_plot(FQR,X,Y);
    end else begin
      BaseX:=FQR^.width+FQR^.border_width*2;
      BaseY:=FQR^.rows+FQR^.border_width*2;
      ErrorCode:=render_plot(FQR,X-(X mod BaseX),(Y-(Y mod BaseY)));
    end;
    if ErrorCode<>1 then begin
      FLastErrorString:=FQR^.errtxt;
      exit;
    end else begin
      FLastErrorString:='';
    end;
  end;
  if Assigned(FQR^.rendered) then begin
    Line:=FQR^.rendered^.lines;
    ClearBackground;
    aTargetCanvas.Brush.Color:=FForegroundColor;
    while Assigned(Line) do begin
      aTargetCanvas.FillRect( round(Line^.x)+aRect.Left,round(Line^.y)+aRect.Top,
                              round(Line^.x+Line^.width)+aRect.Left,round(Line^.y+Line^.length)+aRect.Top);
      Line:=Line^.next;
    end;
  end;
end;

// Erzeuge ein Datamatrix Code in ein vorgegebenes Bitmap
function BuildDatamatrix(Var bmp: TBitmap; s: UTF8String): Integer; stdcall;
Var R: TRect;
begin
  Result := 0;
  If Assigned(bmp) Then
  try
    FText := s;
    R := Rect(0, 0, bmp.Width, bmp.Height);
    bmp.Canvas.Brush.Color:=FBackgroundColor;
    bmp.Canvas.FillRect(R);
    Generate();
    intfPaintOnCanvas(bmp.Canvas, R);
    Generate();
    Result := FQR^.rows;
  finally
    if Assigned(FQR) then begin
      ZBarcode_Delete(FQR);
      FQR:=nil;
    end;
  end;
end;

// Features der DLL, "|" getrennt
function GetDLLFct(): String; cdecl;
Begin
  Result := sVersion + '|BuildDatamatrix';
End;

exports GetDLLFct;
exports BuildDatamatrix;

Begin
  FQR := Nil;
  FBackgroundColor:=clWhite;
  FForegroundColor:=clBlack;
  FStrictSize:=true;
End.
